/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example;

import java.util.Calendar;
import java.util.Date;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.springframework.test.context.ContextConfiguration;
import org.springframework.test.context.TestExecutionListeners;
import org.springframework.test.context.junit4.SpringJUnit4ClassRunner;
import org.springframework.test.context.support.DependencyInjectionTestExecutionListener;

import de.conceptpeople.checkerberry.db.core.CheckerberryDbEnvironment;
import de.conceptpeople.checkerberry.db.core.test.DbTestHandler;
import de.conceptpeople.checkerberry.db.spring.bridge.test.SpringCheckerberryDbEnvironmentCreator;
import de.conceptpeople.checkerberry.db.spring.integration.ApplicationContextResolvingExecutionListener;
import de.conceptpeople.checkerberry.db.spring.integration.TestMethodNameResolvingExecutionListener;

/**
 * In dieser abstrakten Oberklasse wird die checkerberry db-Umgebung erzeugt.
 */
@RunWith(SpringJUnit4ClassRunner.class)
//Spring Konfiguration - dort sind das Include DO und DAO definiert
@ContextConfiguration(locations = { "/application-context-test.xml" })
//Der erste für DependencyInjection, der zweite zur Steuerung von checkerberry
@TestExecutionListeners({ DependencyInjectionTestExecutionListener.class,
     TestMethodNameResolvingExecutionListener.class, ApplicationContextResolvingExecutionListener.class })
public abstract class AbstractExampleTestCase {

    private CheckerberryDbEnvironment environment;

    @Before
    public void setUp() {

        // Checkerberry db-Umgebung wird erzeugt...
        environment = SpringCheckerberryDbEnvironmentCreator.createEnvironment(getBridgeContainerBeanName());
        // ...und initialisiert.
        environment.setUp(this);
    }

    @After
    public void tearDown() {
        environment.tearDown();
        environment = null;
    }

    /**
     * Liefert den Bean-Namen der Bridge-Implementation im Application-Context.
     * Diese Methode kann überschrieben werden, wenn andere Implementationen
     * verwendet werden sollen.
     *
     * @return Bean-Name der zu verwendenden Bridge-Implementation.
     */
    protected String getBridgeContainerBeanName() {
        return "bridgeContainer";
    }

    protected CheckerberryDbEnvironment getEnvironment() {
        return environment;
    }

    protected DbTestHandler getTestHandler() {
        return getEnvironment().getTestHandler();
    }

    /**
     * Erzeugt ein Datumsobjekt.
     *
     * @param day
     *            Tag des Datums.
     * @param month
     *            Monat, wobei Januar = 1, ..., Dezember = 12.
     * @param year
     *            Jahr.
     * @return erzeugtes Datumsobjekt.
     */
    protected final Date createDate(int day, int month, int year) {

        Calendar calendar = Calendar.getInstance();
        calendar.set(year, month - 1, day);

        return calendar.getTime();
    }

    protected void assertNotNull(Object object) {
        Assert.assertNotNull(object);
    }

    protected void assertEquals(int expected, int actual) {
        Assert.assertEquals(expected, actual);
    }

    protected void fail(String msg) {
        Assert.fail(msg);
    }

}
