/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.base;

import org.junit.Test;

import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;
import de.conceptpeople.checkerberry.db.example.dao.UserDao;
import de.conceptpeople.checkerberry.db.example.entities.User;

/**
 * Beispieltest für die einfachsten Fälle.
 */
public class DbExampleBaseIntegrationTest extends AbstractExampleDaoTestCase {

    /**
     * Testet die Methode {@link UserDao#getUser(String, String)}, d.h. ein
     * einfaches Select-Statement.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testGetUser() throws Exception {
        // Die Testdaten unter
        // DbExampleBaseIntegrationTest_testGetUser_initial.xml wurden in der
        // Setup-Phase schon eingespielt, sodass es hier gleich losgehen kann.
        // Die Testdaten sollten natürlich so gewählt sein, dass die
        // Where-Bedingung getestet wird.
        User homer = getUserDao().getUser("Homer", "Simpson");

        // Wir erwarten also, dass wir Homer aus der Datenbank lesen konnten.
        assertNotNull(homer);

        // Und es sollte auch der richtige sein.
        assertEquals(42, homer.getId());

        // Gerade in Hibernate-Umgebungen kann es auch beim Testen von
        // Select-Statements sinnvoll sein, den Datenbankinhalt zu prüfen. Er
        // darf sich durch das Select-Statement nicht verändert haben.
        getTestHandler().assertInitialDataUnchanged();
    }

    /**
     * Testet die Methode {@link UserDao#save(User)}. In diesem Fall ein
     * einfaches Insert-Statement.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testInsertUser() throws Exception {
        // Die Testdaten unter
        // DbExampleBaseIntegrationTest_testInsertUser_initial.xml wurden in der
        // Setup-Phase schon eingespielt, sodass es hier gleich losgehen kann.
        User homer = new User("Homer", "Simpson", createDate(16, 9, 1946));
        homer.setWeight(80.0);

        getUserDao().save(homer);

        // Prüfen, ob die Datenbank jetzt den neuen User enthält.
        getTestHandler().assertEqualsExpected();
    }

    /**
     * Testet die Methode {@link UserDao#saveOrUpdate(User)}. In diesem Fall ein
     * einfaches Update-Statement.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testUpdateUser() throws Exception {
        // Die Testdaten unter DbExampleBaseIntegrationTest_initial.xml wurden
        // in der Setup-Phase schon eingespielt, sodass es hier gleich losgehen
        // kann.
        User homer = getUserDao().getUser("Homer", "Simpson");
        assertNotNull(homer);
        // Donuts und Duff bleiben nicht ohne Folgen: +10 Kilo für Homer...
        homer.setWeight(90.0);

        getUserDao().saveOrUpdate(homer);

        // Prüfen, ob die Datenbank jetzt die erwarteten Daten enthält.
        getTestHandler().assertEqualsExpected();
    }
}
