/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.dump;

import java.io.File;

import org.junit.Test;

import de.conceptpeople.checkerberry.db.example.AbstractExampleTestCase;

/**
 * Beispieltest für die Erzeugung von Datenbank-Dumps im Testdatenformat. Durch
 * die Verwendung von Dumps kann der aktuelle Datenbankinhalt in eine XML-Datei
 * exportiert werden, die dem Testdatenformat entspricht. Auf diese Art und
 * Weise wird u.a. die Erstellung der Testdaten vereinfacht. Darüber hinaus
 * erleichtert das Vorgehen die Fehleranalyse. Tritt z.B. in der Testumgebung
 * ein Fehler auf, kann der Datenbankinhalt als Testdatendatei exportiert werden
 * und in der lokalen Entwicklungsumgebung reproduziert werden.
 */
public class DbExampleDumpIntegrationTest extends AbstractExampleTestCase {

    /**
     * Zeigt die Verwendung der Dump-Methoden.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testDumps() throws Exception {

        // Erzeugt eine XML-Datei mit dem gesamten Datenbankinhalt.
        getTestHandler().dumpDatabase(createTempDumpFile("dump-database"));

        // Erzeugt eine XML-Datei mit dem gesamten Datenbankinhalt, wobei die
        // cacheable Tabelle DISH nicht mit exportiert wird.
        getTestHandler().dumpDatabase(createTempDumpFile("dump-database-without-dish"), true);

        // Erzeugt eine XML-Datei mit der Tabelle USERS.
        getTestHandler().dumpTable(createTempDumpFile("dump-table-users"), "USERS");

        // Erzeugt eine XML-Datei mit den Tabellen USERS und DISH.
        getTestHandler().dumpTables(createTempDumpFile("dump-tables-users-dish"), "USERS", "DISH");

    }

    /**
     * Erzeugt eine temporäre Datei, die am Ende der Tests wieder gelöscht wird.
     *
     * @param name
     *            Name der temporären Datei. Als Dateiendung wird immer XML
     *            verwendet.
     * @return absoluter Name der temporären Datei.
     * @throws Exception
     *             wenn die Datei nicht angelegt werden konnte.
     */
    private String createTempDumpFile(String name) throws Exception {
        File dumpFile = File.createTempFile(name, ".xml");
        // Diese Zeile auskommentieren, um einen Blick in die Datei zu werfen.
        dumpFile.deleteOnExit();
        return dumpFile.getAbsolutePath();
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected String getBridgeContainerBeanName() {

        // Verwendet den Bridge-Container, der die Tabelle DISH als cacheable
        // markiert.
        return "bridgeContainerWithCaching";
    }
}
