/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.parameter;

import org.junit.Before;
import org.junit.Test;

import de.conceptpeople.checkerberry.db.bridge.context.DatabaseDescription;
import de.conceptpeople.checkerberry.db.bridge.context.DatabaseTableDescription;
import de.conceptpeople.checkerberry.db.example.AbstractExampleDaoTestCase;
import de.conceptpeople.checkerberry.db.example.dao.UserDao;
import de.conceptpeople.checkerberry.db.example.entities.User;

/**
 * Dieser Beispieltest enthält die Verwendung der Parameter.
 */
public class DbExampleParameterIntegrationTest extends AbstractExampleDaoTestCase {

    @Before
    @Override
    public void setUp() {
        super.setUp();

        // In der Standardkonfiguration ist die Spalte ID vom Vergleich
        // ausgeschlossen. Das ist in der Regel auch sinnvoll, da es ein
        // technischer Wert ist, der erst beim Anlegen eines neuen Datensatzes
        // in der Datenbank angelegt wird. Hier nehmen wir die Spalte wieder in
        // den Vergleich auf, um den Umgang mit dynamischen Werten zu
        // demonstrieren, die zum Zeitpunkt der Testerstellung noch nicht
        // feststehen.
        removeAllExcludedColumnsFromUsersTable();
    }

    /**
     * Testet die Methode {@link UserDao#save(User)}. In diesem Fall ein
     * einfaches Insert-Statement.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testInsertUser() throws Exception {

        // Die Testdaten unter
        // DbExampleParameterIntegrationTest_testInsertUser_initial.xml wurden
        // in der
        // Setup-Phase schon eingespielt, sodass es hier gleich losgehen kann.
        User homer = new User("Homer", "Simpson", createDate(16, 9, 1946));
        homer.setWeight(80.0);

        getUserDao().save(homer);

        // Nach dem Speichern von Homer in der Datenbank wurde die ID des
        // Datensatzes in das User-Objekt eingetragen. Die ID wird dann in einem
        // Parameter gespeichert, der auch in den Testdaten verwendet werden
        // kann.
        getTestHandler().getParameterContext().addParameter("homer.id", homer.getId());

        // Prüfen, ob die Datenbank jetzt den neuen User enthält. Die ID kann
        // ebenfalls geprüft werden, da sie als Parameter gesetzt wurde.
        getTestHandler().assertEqualsExpected();
    }

    /**
     * Entfernt alle ausgeschlossenen Spalten von der Tabelle USERS, d.h.
     * dadurch werden alle Spalten bei der Überprüfung berücksichtigt.
     */
    private void removeAllExcludedColumnsFromUsersTable() {
        DatabaseDescription databaseDescription = getTestHandler().getDatabaseDescription();
        DatabaseTableDescription usersDescription = databaseDescription.getTableDescription("USERS");

        // Die Liste der ausgeschlossenen Spalten wird überschrieben.
        usersDescription.clearExcludedColumns();
    }
}
