/*
 * checkerberry test center - https://www.checkerberry.de
 *
 * Copyright (c) 2010 ConceptPeople consulting gmbh
 *
 * Lizenziert unter der EUPL, Version 1.1 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/software/page/eupl5
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.conceptpeople.checkerberry.db.example.report;

import java.io.File;

import org.junit.Test;

import de.conceptpeople.checkerberry.db.example.AbstractExampleTestCase;

/**
 * Beispieltest für die Verwendung der Reports. Die Reports vergleichen immer
 * zwei Datenmengen miteinander. Zur Auswahl stehen die drei Datenmengen:
 * initialen und erwarteten Testdaten, sowie die tatsächlichen Daten in der
 * Datenbank.
 */
public class DbExampleReportIntegrationTest extends AbstractExampleTestCase {

    /**
     * Vergleicht die erwarteten Testdaten mit den tatsächlichen Daten. Dieser
     * Report dient dazu, alle Abweichungen auf einen Blick zu sehen, um die
     * Ursache für die Abweichungen schneller zu finden.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testDiffReport() throws Exception {

        getTestHandler().createDiffReport(createTempReportFile("diff-report"));
    }

    /**
     * Vergleicht die initialen mit den erwarteten Testdaten. Dieser Report
     * dient zur Anzeige der erwarteten Änderungen in den Testdaten, die durch
     * die zu testende Funktion hervorgerufen werden soll. Auf diese Art und
     * Weise lässt sich die Korrektheit der Testdaten leichter verifizieren.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testExpectationReport() throws Exception {

        getTestHandler().createExpectationReport(createTempReportFile("expectation-report"));
    }

    /**
     * Vergleicht die initialen Testdaten mit den tatsächlichen Daten. Dieser
     * Report dient zur Analyse, welche Änderungen durch die zu testende
     * Funktion hervorgerufen werden. Auf diese Art und Weise lässt sich die
     * Funktionsweise unbekannter Services o.ä. gut analysieren.
     *
     * @throws Exception
     *             wenn ein Fehler aufgetreten ist.
     */
    @Test
    public void testProcessAnalysisReport() throws Exception {

        // Bei allen Reports kann man die Endungen der Testdaten angeben. Der
        // Default hier ist "initial". Durch die Änderung auf "result" passiert
        // folgendes: Die initialen Testdaten werden eingespielt und stellen den
        // tatsächlichen Datenbestand dar. Durch die Angabe der Endung "result"
        // im Report, werden somit die Testdaten aus den erwarteten Testdaten
        // als initiale Testdaten betrachtet, sodass der Report die Abweichungen
        // von der erwarteten zu den initialen Testdaten anzeigt. Alles klar?
        getTestHandler().createProcessAnalysisReport("result",
                createTempReportFile("process-analysis-report"));
    }

    /**
     * Erzeugt eine temporäre Datei, die am Ende der Tests wieder gelöscht wird.
     *
     * @param name
     *            Name der temporären Datei. Als Dateiendung wird immer HTML
     *            verwendet.
     * @return absoluter Name der temporären Datei.
     * @throws Exception
     *             wenn die Datei nicht angelegt werden konnte.
     */
    private String createTempReportFile(String name) throws Exception {
        File reportFile = File.createTempFile(name, ".html");
        // Diese Zeile auskommentieren, um einen Blick in die Datei zu werfen.
        reportFile.deleteOnExit();
        return reportFile.getAbsolutePath();
    }

}
